/***********************************************************************
IR_38K_TX
IR remote control for fan heater for some type of IR receiver 
Author: Giuseppe Aquilani - IK0JRE
Version: 0.3 November 2012


The sketch transmit with 38KHz modulation with standard product code

You can change prod. code or key code in setup() function

 
 
                             IR LED
 
                      C           | /|     10 ohm
                       ___________|/ |___/\/\/\/\_________ + 5V
 Arduino              /           |\ |
 pin 10            | /            | \| 
                B  |/
 o-------\/\/\/\---|   BC337 or Standard NPN
      |   470 ohm  |\
      |            | \ 
      <               \ E
      >  1K           |
      <               |
      |               |   
     ---             --- 
     |||             |||  GND
                 
            
                 
     IR Protocol
All code was tested with arduino uno

    0                      1
         ___            _______
|*******|           ***|       |
|*******|           ***|       |
|_______|           ___|       |
   bit   space     space  bit
 
 ** = 38KHz modulation
 bit 1 has modulation on space time (USSPACE)
 bit 0 has modulation on bit time (USLENBIT = USSPACE * 3)
                               
***********************************************************************/
#include <arduino.h>

#define BZOUT   5     //buzzer driven to + polarity 
#define TXOUT   10    //drive IRLED with NPN transistor
#define IRINPUT 11    //input for phototransistor decoder ex. TSOP4838
#define K1INPUT 12    //button normal status high w pull-up 10k resistor
#define K5INPUT 9     //button normal status high w pull-up 10k resistor

#define USSPACE   430  //uS valid space time
#define USLENBIT  (USSPACE * 3) //uS valid bit time
#define DL38KHZ   10   //uS half pulse 38Khz (((1/0.038)/2) - code delay)
#define DLTRAIN   5    //delay betwen two pulse train 
#define MAXKEY    8    //max bit key code 

byte prodcode;
byte keycode[MAXKEY];

unsigned int spulse[255];
byte markspace[255];
byte cpulse;
byte canread;
/***********************************************************************
* standard arduino setup function
***********************************************************************/
void setup()
{
  cpulse=0;
  canread=0;
  
  for(int i=0;i<255;i++){
      spulse[i]=0;
      markspace[i]=0;
      }
 // pinMode(BZOUT,OUTPUT);

  pinMode(IRINPUT,INPUT);
  pinMode(TXOUT,OUTPUT);
  pinMode(K1INPUT,INPUT);
  pinMode(K5INPUT,INPUT);
  
  Serial.begin(9600);

  //digitalWrite(BZOUT,LOW);
  digitalWrite(TXOUT,LOW);
  
 /***********************************************************************
  * coding key protocol
  ***********************************************************************/
 /* bit send order is from RIGHT to left (right bit (LSB) is first bit to send) */
  prodcode = 0x0C;  //B1100;
  
  keycode[0]=0x7F;  //B01111111; STOP FAN
  keycode[1]=0xBF;  //B10111111; START/STOP LIGHT
  keycode[2]=0xDF;  //B11011111; START FAN LOW
  keycode[3]=0xEF;  //B11101111; START FAN MID
  keycode[4]=0xF7;  //B11110111; START FAN HIGH
  keycode[5]=0xFB;  //B11111011; START TIMER 1H
  keycode[6]=0x3D;  //B00111101; START TIMER 2H
  keycode[7]=0x9D;  //B10011101; START TIMER 4H
 }
/***********************************************************************
* void BeepOUT()
* only for test frequency
***********************************************************************/
void BeepOUT()
{
  int i=0;

  for(i=0;i<100;i++)
  {
    digitalWrite(BZOUT,HIGH);
    delayMicroseconds(500); 
    digitalWrite(BZOUT,LOW);
    delayMicroseconds(500); 
  }
}
/***********************************************************************
* PULSE38K(int duration)
* pulse 38KHz mod for time needed
***********************************************************************/
void PULSE38K(int duration)
{
  unsigned long Tstart;
  unsigned long Tstop;
  Tstart=micros();
     do{
       digitalWrite(TXOUT,LOW);
       delayMicroseconds(DL38KHZ); 
       digitalWrite(TXOUT,HIGH);
       delayMicroseconds(DL38KHZ); 
      }while(Tstop=micros()-Tstart<duration); 
}
/***********************************************************************
* void TX38KHIGH()
* send bit logic high w 38KHz mod. 
***********************************************************************/
void TX38KHIGH()
{
  digitalWrite(TXOUT,LOW);
  PULSE38K(USSPACE);
  delayMicroseconds(USLENBIT);
  digitalWrite(TXOUT,LOW);
}
/***********************************************************************
* TX38KLOW()
* send bit logic low  w 38KHz mod.
***********************************************************************/
void TX38KLOW()
{
  digitalWrite(TXOUT,HIGH);
  PULSE38K(USLENBIT);
  digitalWrite(TXOUT,HIGH);
  delayMicroseconds(USLENBIT);
}
/***********************************************************************
* sendKey(int key)
* accept key from 0 to 7
* send 4 bit prod code before next 8 bit key code 
***********************************************************************/
void sendKey(int key)
{
  if(key > MAXKEY-1) return;
  int keyval;
  int i;
  
    for(i = 0;i<=3;i++)
     {
      keyval = (prodcode >> i); 
      if((keyval & 0x01) == 0)
        TX38KLOW();
      else
        TX38KHIGH();
     } //end send prod code
  
 
   for( i = 0;i<=(MAXKEY-1);i++)
     {
      keyval = (keycode[key] >> i); 
      if((keyval & 0x01) == 0)
        TX38KLOW();
      else
        TX38KHIGH();
      }//end send key code
  
   delay(DLTRAIN);
}
/***********************************************************************
* TestK1TX()
* was used for quicky and dirty test
***********************************************************************
void TestK1TX()
{
 // prod. code 
 TX38KLOW();
  TX38KLOW();
  TX38KHIGH();
  TX38KHIGH();
 
 // key code
  TX38KHIGH();
  TX38KHIGH();
  TX38KHIGH();
  TX38KHIGH();
  TX38KHIGH();
  TX38KHIGH();
  TX38KHIGH();
  TX38KLOW();
  
  delay(DLTRAIN);
}*/

/**********************************************
* readIR()
* only for test 
* it should give bit and space duration time
* but I don't have got a time
**********************************************/
void readIR()
{
  int dSer;
  int cend=0;
  boolean error=0;
  unsigned long Tstart;
  unsigned long Tstop;
  byte cRead=0;
  cpulse=0;   //note: we undestand cpulse can to assume more than 255 values but we do nothing
 
  do{
   if(( cRead=digitalRead(IRINPUT)) == 0)
    {
     Tstart=micros();
      do{
         Tstop=(micros()-Tstart);
         cRead=digitalRead(IRINPUT);
        }while(cRead != 1);
       markspace[cpulse]=0;
      spulse[cpulse++]=Tstop;
    }
   else  
     {
       Tstart=micros();
       do{
         cRead=digitalRead(IRINPUT);
         Tstop = micros()-Tstart;
          if(Tstop > 2000) //if HIGH level duration pulse > 2000 uS nothing to do
            { 
              cend =1;
              cRead =0;
              canread++;  //we want discard first train
             }
         }while(cRead != 0);
        markspace[cpulse]=1;
        spulse[cpulse++]=Tstop;
        
        if(cpulse == 255){error=cend=1;}
        
       } //end if
    }while(cend!=1);

  //write to serial a result
  //s indicate space (logic level 1)
  //m indicate mark (logic level 0)
  //follow is time in uS
  
  //we show train start from 1 pulse to 255 AND if it is 2nd TRAIN 
  if((cpulse>1) && (canread > 2))
    {
     for(int i=0;i<cpulse;i++)
        {
         if(markspace[i]==1)
            Serial.print("s");
         else
            Serial.print("m");
           
         Serial.print(spulse[i]);
         Serial.print(" ");
        }//end for
       Serial.println(error);
       canread=0;  //train read reset counter
     } //end if
}
/**********************************************
 * standard arduino loop
 **********************************************/
 void loop()
 {
 int dSer=0;

 if(digitalRead(K1INPUT)==0)
     sendKey(0); //stop
    
 if(digitalRead(K5INPUT)==0)
    sendKey(4); //start hi speed
    
     
 //for test without switch you can send key 0 to 7
 //there is not control if you type other key  
   if(Serial.available()>0){
      dSer=Serial.read()-48;
      sendKey(dSer);
    }
   if(digitalRead(IRINPUT)==0)readIR();
 }

